@@ -6,15 +6,33 @@ from .common import *
 
 
 # S - PARAMETERS #
+class SSMParameter(ssm.Parameter):
+    Type = "String"
+
+
 class Parameter(Parameter):
     def __init__(self, title, **kwargs):
         super().__init__(title, **kwargs)
-        self.Type = 'String'
-        self.Default = ''
+        self.Type = "String"
+        self.Default = ""
+
+        # Create SSM Parameter for EnvAppXVersion Params to have history of application versions
+        if title.startswith("EnvApp") and title.endswith("Version"):
+            add_obj(
+                [
+                    get_condition(title, "not_equals", "", nomap=True),
+                    SSMParameter(
+                        f"SSMParameter{title}",
+                        Condition=title,
+                        Name=Sub(
+                            "/EnvAppVersions/${EnvRole}/${AWS::StackName}/%s" % title
+                        ),
+                        Value=Ref(title),
+                    ),
+                ]
+            )
 
 
-class SSMParameter(ssm.Parameter):
-    Type = 'String'
 # E - PARAMETERS #
 
 
@@ -23,18 +41,18 @@ def stack_add_res():
         # Automatically create override conditions for parameters
         if not n.startswith(PARAMETERS_SKIP_OVERRIDE_CONDITION):
 
-            if n.endswith('InstanceType'):
-                default = 'default'
-            elif n == 'SecurityGroups':
+            if n.endswith("InstanceType"):
+                default = "default"
+            elif n == "SecurityGroups":
                 default = cfg.SECURITY_GROUPS_DEFAULT
             else:
-                default = ''
+                default = ""
 
-            condition = {
-                f'{n}Override': Not(Equals(Select(0, Ref(n)), default))
-            } if v.Type == 'CommaDelimitedList' else {
-                f'{n}Override': Not(Equals(Ref(n), default))
-            }
+            condition = (
+                {f"{n}Override": Not(Equals(Select(0, Ref(n)), default))}
+                if v.Type == "CommaDelimitedList"
+                else {f"{n}Override": Not(Equals(Ref(n), default))}
+            )
 
             add_obj(condition)
         # End
@@ -80,21 +98,35 @@ def add_objoutput(res):
     except Exception:
         pass
     else:
+        mapname = iboxprops["MAP"][res.title]
+
         if isinstance(res.Value, str):
-            mapname = iboxprops['MAP'][res.title]
             join_list = []
             for n in res.Value.split():
                 n = n.strip()
-                if n.startswith('${'):
-                    n = n.strip('${}')
-                    obj = iboxprops[f'{mapname}{n}'][0]
-                    propname = iboxprops[f'{mapname}{n}'][1]
+                if n.startswith("${"):
+                    n = n.strip("${}")
+                    obj = iboxprops[f"{mapname}{n}"][0]
+                    propname = iboxprops[f"{mapname}{n}"][1]
                     n = getattr(obj, propname)
                 join_list.append(n)
 
-            res.Value = Join('', join_list)
+            res.Value = Join("", join_list)
+        else:
+            # output use relative resource condition if do not have one
+            # i do this only if Value is not a string
+            # i need Condition only if using Ref for getting the resource
+            try:
+                res.Condition
+            except Exception:
+                try:
+                    cond = iboxprops[f"{mapname}Condition"][0].Condition
+                except Exception:
+                    pass
+                else:
+                    res.Condition = cond
 
-        del res.properties['IBOX_PROPS']
+        del res.properties["IBOX_PROPS"]
 
 
 def do_no_override(action):
@@ -104,10 +136,20 @@ def do_no_override(action):
         cfg.no_override = False
 
 
-def get_endvalue(param, ssm=False, condition=False, nocondition=False,
-                 nolist=False, inlist=False, split=False, issub=False,
-                 strout=False, fixedvalues=None, mapinlist=False,
-                 resname=None):
+def get_endvalue(
+    param,
+    ssm=False,
+    condition=False,
+    nocondition=False,
+    nolist=False,
+    inlist=False,
+    split=False,
+    issub=False,
+    strout=False,
+    fixedvalues=None,
+    mapinlist=False,
+    no_check_for_mappedvalues=False,
+):
     if not fixedvalues:
         # set default if not defined
         fixedvalues = cfg.fixedvalues
@@ -115,43 +157,45 @@ def get_endvalue(param, ssm=False, condition=False, nocondition=False,
     def _get_overridevalue(param, value, condname=None):
         if param not in cfg.Parameters and condname in cfg.Parameters:
             param = condname
-        if (cfg.no_override is False and param in cfg.Parameters and not
-                param.startswith(PARAMETERS_SKIP_OVERRIDE_CONDITION)):
-            override_value = If(f'{param}Override', Ref(param), value)
+        if (
+            cfg.no_override is False
+            and param in cfg.Parameters
+            and not param.startswith(PARAMETERS_SKIP_OVERRIDE_CONDITION)
+        ):
+            override_value = If(f"{param}Override", Ref(param), value)
         else:
             override_value = value
 
         return override_value
 
     def _get_value():
-        if resname:
-            IBOX_RESNAME = resname
         # if param in fixedvalues means its value do not changes
-        # based on Env/Region so hardcode the value in json, ...
+        # based on Env/Region so hardcode the value.
         if param in fixedvalues:
             value = fixedvalues[param]
             # check if value start with method and use eval to run code
             if isinstance(value, list):
-                scope_g = globals()
-                scope_l = locals()
                 value = [
-                    eval(r, scope_g, scope_l) if str(r).startswith(
-                        cfg.EVAL_FUNCTIONS_IN_CFG) else r for r in value
+                    eval(r) if str(r).startswith(cfg.EVAL_FUNCTIONS_IN_CFG) else r
+                    for r in value
                 ]
             if isinstance(value, str):
                 value = (
-                    eval(value.replace('\n', ''))
+                    eval(value.replace("\n", ""))
                     if value.startswith(cfg.EVAL_FUNCTIONS_IN_CFG)
-                    else value)
+                    else value
+                )
         # ... otherway use mapping
+        elif no_check_for_mappedvalues or param in cfg.mappedvalues:
+            value = FindInMap(Ref("EnvShort"), Ref("AWS::Region"), param)
         else:
-            value = FindInMap(Ref('EnvShort'), Ref('AWS::Region'), param)
+            raise ValueError(f"{param} not present in mappedvalues.")
 
         if strout is True and isinstance(value, int):
             value = str(value)
 
         if nolist is True and isinstance(value, list):
-            value = ','.join(value)
+            value = ",".join(value)
 
         if issub:
             value = Sub(value)
@@ -175,14 +219,14 @@ def get_endvalue(param, ssm=False, condition=False, nocondition=False,
 
         value = If(
             condname,
-            value if condition else Ref('AWS::NoValue'),
-            Ref('AWS::NoValue') if condition else value,
+            value if condition else Ref("AWS::NoValue"),
+            Ref("AWS::NoValue") if condition else value,
         )
     else:
         value = _get_overridevalue(param, value)
 
     if split is not False:
-        value = Select(split, Split(',', value))
+        value = Select(split, Split(",", value))
 
     if inlist is not False:
         value = Select(inlist, value)
@@ -190,16 +234,14 @@ def get_endvalue(param, ssm=False, condition=False, nocondition=False,
     return value
 
 
-def get_expvalue(param, stack=False, prefix=''):
-    v = ''
+def get_expvalue(param, stack=False, prefix=""):
+    v = ""
     if stack:
         v = ImportValue(
-            Sub('%s-${%s}' % (param, stack), **{stack: get_endvalue(stack)})
+            Sub("%s-${%s}" % (param, stack), **{stack: get_endvalue(stack)})
         )
     elif not isinstance(param, str):
-        v = ImportValue(
-            Sub('%s${ImportName}' % prefix, **{'ImportName': param})
-        )
+        v = ImportValue(Sub("%s${ImportName}" % prefix, **{"ImportName": param}))
     else:
         v = ImportValue(param)
 
@@ -208,29 +250,26 @@ def get_expvalue(param, stack=False, prefix=''):
 
 def get_subvalue(substring, subvar, stack=False):
     submap = {}
-    found = substring.find('${')
+    found = substring.find("${")
 
     while found != -1:
         posindex = found + 2
         myindex = substring[posindex]
         mytype = substring[posindex + 1]
         # M = Mapped, E = Exported
-        if myindex.isdigit() and mytype in ['M', 'E']:
-            listitem = (
-                subvar[int(myindex) - 1] if isinstance(subvar, list)
-                else subvar)
-            stackitem = (
-                stack[int(myindex) - 1] if isinstance(stack, list)
-                else stack)
-            if mytype == 'M':
+        if myindex.isdigit() and mytype in ["M", "E"]:
+            listitem = subvar[int(myindex) - 1] if isinstance(subvar, list) else subvar
+            stackitem = stack[int(myindex) - 1] if isinstance(stack, list) else stack
+            if mytype == "M":
                 submap[listitem] = get_endvalue(listitem)
             else:
                 submap[listitem] = get_expvalue(listitem, stackitem)
 
             substring = substring.replace(
-                '${%s%s}' % (myindex, mytype), '${%s}' % listitem)
+                "${%s%s}" % (myindex, mytype), "${%s}" % listitem
+            )
 
-        found = substring.find('${', posindex)
+        found = substring.find("${", posindex)
 
     v = Sub(substring, **submap)
 
@@ -240,12 +279,12 @@ def get_subvalue(substring, subvar, stack=False):
 def get_resvalue(resname, propname):
     res = cfg.Resources[resname]
 
-    loc = propname.find('.')
+    loc = propname.find(".")
     while loc > 0:
         prop = propname[0:loc]
         res = getattr(res, prop)
-        propname = propname[loc + 1:]
-        loc = propname.find('.')
+        propname = propname[loc + 1 :]
+        loc = propname.find(".")
 
     return getattr(res, propname)
 
@@ -256,51 +295,52 @@ def get_dictvalue(key):
     elif isinstance(key, dict):
         value = {i: get_dictvalue(k) for i, k in key.items()}
     else:
-        value = eval(key) if key.startswith(
-            cfg.EVAL_FUNCTIONS_IN_CFG) else key
+        value = eval(key) if key.startswith(cfg.EVAL_FUNCTIONS_IN_CFG) else key
 
     return value
 
 
-def get_condition(cond_name, cond, value2, key=None, OrExtend=[],
-                  mapinlist=False, nomap=None):
+def get_condition(
+    cond_name, cond, value2, key=None, OrExtend=[], mapinlist=False, nomap=None
+):
     # record current state
     override_state = cfg.no_override
     do_no_override(True)
 
     key_name = key if key else cond_name
     if isinstance(key, FindInMap):
-        map_name = key.data['Fn::FindInMap'][0]
-        key_name = key.data['Fn::FindInMap'][1]
-        value_name = key.data['Fn::FindInMap'][2]
+        map_name = key.data["Fn::FindInMap"][0]
+        key_name = key.data["Fn::FindInMap"][1]
+        value_name = key.data["Fn::FindInMap"][2]
         if not value_name and cond_name:
             value_name = cond_name
 
         value1_param = FindInMap(map_name, Ref(key_name), value_name)
         value1_map = FindInMap(map_name, get_endvalue(key_name), value_name)
     elif isinstance(key, Select):
-        select_index = key.data['Fn::Select'][0]
-        select_list = key.data['Fn::Select'][1]
+        select_index = key.data["Fn::Select"][0]
+        select_list = key.data["Fn::Select"][1]
 
-        if 'Fn::Split' in select_list.data:
-            split_sep = select_list.data['Fn::Split'][0]
-            key_name = select_list.data['Fn::Split'][1]
+        if "Fn::Split" in select_list.data:
+            split_sep = select_list.data["Fn::Split"][0]
+            key_name = select_list.data["Fn::Split"][1]
             select_value_param = Split(split_sep, Ref(key_name))
             select_value_map = Split(split_sep, get_endvalue(key_name))
         else:
             select_value_param = select_list
             select_value_map = get_endvalue(select_list)
 
-        value1_param = Select(
-            select_index, select_value_param)
-        value1_map = Select(
-            select_index, select_value_map)
+        value1_param = Select(select_index, select_value_param)
+        value1_map = Select(select_index, select_value_map)
     else:
         value1_param = Ref(key_name)
         # Used new param "mapinlist" when you have a mapped value in a list
         # but multiple values as override parameters
         if mapinlist:
             value1_map = get_endvalue(mapinlist[0], mapinlist=mapinlist[1])
+        elif nomap:
+            # if nomap need to avoid checking that value does exist in mappedvalues
+            value1_map = get_endvalue(key_name, no_check_for_mappedvalues=True)
         else:
             value1_map = get_endvalue(key_name)
 
@@ -311,17 +351,19 @@ def get_condition(cond_name, cond, value2, key=None, OrExtend=[],
     eq_param = Equals(value1_param, value2)
     eq_map = Equals(value1_map, value2)
 
-    if cond == 'equals':
+    if cond == "equals":
         cond_param = eq_param
         cond_map = eq_map
-    elif cond == 'not_equals':
+    elif cond == "not_equals":
         cond_param = Not(eq_param)
         cond_map = Not(eq_map)
 
-    if (key_name in cfg.Parameters and
-            not key_name.startswith(PARAMETERS_SKIP_OVERRIDE_CONDITION) and
-            not nomap):
-        key_override = f'{key_name}Override'
+    if (
+        key_name in cfg.Parameters
+        and not key_name.startswith(PARAMETERS_SKIP_OVERRIDE_CONDITION)
+        and not nomap
+    ):
+        key_override = f"{key_name}Override"
         condition = Or(
             And(
                 Condition(key_override),
@@ -330,10 +372,10 @@ def get_condition(cond_name, cond, value2, key=None, OrExtend=[],
             And(
                 Not(Condition(key_override)),
                 cond_map,
-            )
+            ),
         )
         if OrExtend:
-            condition.data['Fn::Or'].extend(OrExtend)
+            condition.data["Fn::Or"].extend(OrExtend)
     elif nomap:
         condition = cond_param
     else:
@@ -346,13 +388,13 @@ def get_condition(cond_name, cond, value2, key=None, OrExtend=[],
 
 
 def import_lambda(name):
-    TK_IN_LBD = 'IBOX_CODE_IN_LAMBDA'
+    TK_IN_LBD = "IBOX_CODE_IN_LAMBDA"
     parent_dir_name = os.path.dirname(os.path.realpath(__file__))
-    lambda_file = os.path.join(os.getcwd(), 'lib/lambdas/%s.code' % name)
+    lambda_file = os.path.join(os.getcwd(), "lib/lambdas/%s.code" % name)
     if not os.path.exists(lambda_file):
-        lambda_file = os.path.join(parent_dir_name, 'lambdas/%s.code' % name)
+        lambda_file = os.path.join(parent_dir_name, "lambdas/%s.code" % name)
     try:
-        with open(lambda_file, 'r') as f:
+        with open(lambda_file, "r") as f:
             fdata = f.read()
 
             try:
@@ -362,7 +404,7 @@ def import_lambda(name):
                 code = fdata
 
             if len(code) > 4096:
-                logging.warning(f'Inline lambda {lambda_file} > 4096')
+                logging.warning(f"Inline lambda {lambda_file} > 4096")
 
             code_lines = code.splitlines(keepends=True)
 
@@ -376,35 +418,39 @@ def import_lambda(name):
                 elif TK_IN_LBD in x:
                     # parse minified code
                     tks = x.split(TK_IN_LBD)
-                    file_lines.extend([
-                        f'{tks[0]}', eval(tks[1]), tks[2]])
+                    file_lines.extend([f"{tks[0]}", eval(tks[1]), tks[2]])
                     continue
                 else:
-                    value = ''.join(x)
+                    value = "".join(x)
 
                 file_lines.append(value)
 
-            return(file_lines)
+            return file_lines
 
     except IOError:
-        logging.warning(f'Lambda code {name} not found')
+        logging.warning(f"Lambda code {name} not found")
         exit(1)
 
 
-def auto_get_props(obj, mapname=None, key=None, rootdict=None):
-    # IBOX_RESNAME can be used in yaml
+def auto_get_props(
+    obj, mapname=None, key=None, rootdict=None, indexname="", remapname=None
+):
+    # IBOX_RESNAME can be used in yaml and resolved inside get_endvalue
+    global IBOX_RESNAME, IBOX_MAPNAME, IBOX_INDEXNAME, IBOX_REMAPNAME
     IBOX_RESNAME = obj.title
     IBOX_MAPNAME = mapname
+    IBOX_REMAPNAME = remapname
+    IBOX_INDEXNAME = indexname
 
     # create a dict where i will put all property with a flat hierarchy
     # with the name equals to the mapname and the relative value.
     # Later i will assign this dict to the relative output object using
     # IBOX_OUTPUT
-    IBOX_PROPS = {'MAP': {}}
+    IBOX_PROPS = {"MAP": {}}
 
     def _iboxif(if_wrapper, mapname, value):
-        condname = if_wrapper[0].replace('IBOX_MAPNAME_', mapname)
-        condname = condname.replace('_', IBOX_RESNAME)
+        condname = if_wrapper[0].replace("IBOX_MAPNAME_", mapname)
+        condname = condname.replace("_", IBOX_RESNAME)
         condvalues = []
         for i in if_wrapper[1:3]:
             if isinstance(i, str) and i.startswith(cfg.EVAL_FUNCTIONS_IN_CFG):
@@ -413,7 +459,7 @@ def auto_get_props(obj, mapname=None, key=None, rootdict=None):
                 v = i
             condvalues.append(v)
 
-        if condvalues[0] == 'IBOX_IFVALUE':
+        if condvalues[0] == "IBOX_IFVALUE":
             value = If(condname, value, condvalues[1])
         else:
             value = If(condname, condvalues[0], value)
@@ -422,24 +468,24 @@ def auto_get_props(obj, mapname=None, key=None, rootdict=None):
 
     def _get_obj(obj, key, obj_propname, mapname):
         props = obj.props
-        mapname_obj = f'{mapname}{obj_propname}'
+        mapname_obj = f"{mapname}{obj_propname}"
 
         def _get_obj_tags():
             prop_list = []
             for k, v in key[obj_propname].items():
                 try:
-                    v['Value']
+                    v["Value"]
                 except Exception:
                     pass
                 else:
-                    v['Value'] = get_endvalue(f'{mapname_obj}{k}Value')
+                    v["Value"] = get_endvalue(f"{mapname_obj}{k}Value")
 
                 try:
-                    if_wrapper = v['IBOX_IF']
+                    if_wrapper = v["IBOX_IF"]
                 except Exception:
                     pass
                 else:
-                    del v['IBOX_IF']
+                    del v["IBOX_IF"]
                     v = _iboxif(if_wrapper, mapname, v)
 
                 prop_list.append(v)
@@ -450,7 +496,7 @@ def auto_get_props(obj, mapname=None, key=None, rootdict=None):
             return obj_tags
 
         # trick (bad) to detect attributes as CreationPolicy and UpdatePolicy
-        if obj_propname in ['CreationPolicy', 'UpdatePolicy']:
+        if obj_propname in ["CreationPolicy", "UpdatePolicy"]:
             prop_class = getattr(pol, obj_propname)
         else:
             prop_class = props[obj_propname][0]
@@ -464,41 +510,43 @@ def auto_get_props(obj, mapname=None, key=None, rootdict=None):
             else:
                 prop_obj = prop_class()
 
-            if prop_class.__bases__[0].__name__ in ['AWSProperty',
-                                                    'AWSAttribute']:
+            if prop_class.__bases__[0].__name__ in ["AWSProperty", "AWSAttribute"]:
                 _populate(prop_obj, key=key[obj_propname], mapname=mapname_obj)
-            elif prop_class.__name__ == 'dict':
+            elif prop_class.__name__ == "dict":
                 prop_obj = get_dictvalue(key[obj_propname])
-            elif prop_class.__name__ == 'Tags':
+            elif prop_class.__name__ == "Tags":
                 prop_obj = _get_obj_tags()
 
             return prop_obj
 
-        elif isinstance(prop_class, tuple) and any(n in prop_class for n in [
-                Tags, asgTags]):
+        elif isinstance(prop_class, tuple) and any(
+            n in prop_class for n in [Tags, asgTags]
+        ):
             prop_obj = _get_obj_tags()
 
             return prop_obj
 
-        elif (isinstance(prop_class, list) and
-                isinstance(prop_class[0], type) and
-                prop_class[0].__bases__[0].__name__ == 'AWSProperty'):
+        elif (
+            isinstance(prop_class, list)
+            and isinstance(prop_class[0], type)
+            and prop_class[0].__bases__[0].__name__ == "AWSProperty"
+        ):
             prop_list = []
             prop_class = props[obj_propname][0][0]
             for o, v in key[obj_propname].items():
-                if o == 'IBOX_IF':
+                if o == "IBOX_IF":
                     # element named IBOX_IF must no be parsed
                     # is needed for wrapping whole returned obj in _populate
                     continue
                 name_o = str(o)
-                mapname_o = f'{mapname_obj}{name_o}'
+                mapname_o = f"{mapname_obj}{name_o}"
                 prop_obj = prop_class()
 
                 _populate(prop_obj, key=v, mapname=mapname_o)
 
                 # trick to wrapper single obj in If Condition
                 try:
-                    if_wrapper = v['IBOX_IF']
+                    if_wrapper = v["IBOX_IF"]
                 except Exception:
                     prop_list.append(prop_obj)
                 else:
@@ -511,54 +559,47 @@ def auto_get_props(obj, mapname=None, key=None, rootdict=None):
             mapname = obj.title
         if rootdict:
             key = rootdict
-            mapname = ''
+            mapname = ""
         if not key:
             key = getattr(cfg, mapname)
 
         def _try_PCO_in_obj(key):
             def _parameter(k):
                 for n, v in k.items():
-                    n = n.replace('_', IBOX_RESNAME)
+                    n = n.replace("{IBOX_INDEXNAME}", IBOX_INDEXNAME)
+                    n = n.replace("_", IBOX_RESNAME)
                     parameter = Parameter(n)
                     _populate(parameter, rootdict=v)
                     add_obj(parameter)
 
             def _condition(k):
-                # this is needed or eval do not find IBOX_RESNAME??
-                IBOX_RESNAME
-                IBOX_MAPNAME
                 for n, v in k.items():
                     if IBOX_MAPNAME:
-                        n = n.replace('IBOX_MAPNAME', IBOX_MAPNAME)
-                    n = n.replace('_', IBOX_RESNAME)
+                        n = n.replace("IBOX_MAPNAME", IBOX_MAPNAME)
+                    n = n.replace("{IBOX_INDEXNAME}", IBOX_INDEXNAME)
+                    n = n.replace("_", IBOX_RESNAME)
                     condition = {n: eval(v)}
                     add_obj(condition)
 
-            def _output_no_more_used(k):
-                for n, v in k.items():
-                    n = n.replace('_', IBOX_RESNAME)
-                    output = Output(n)
-                    _populate(output, rootdict=v)
-                    add_obj(output)
-
             def _output(k):
                 for n, v in k.items():
-                    n = n.replace('_', IBOX_RESNAME)
+                    n = n.replace("{IBOX_INDEXNAME}", IBOX_INDEXNAME)
+                    n = n.replace("_", IBOX_RESNAME)
                     output = Output(n)
                     _populate(output, rootdict=v)
                     # alter troposphere obj and add IBOX property
-                    output.props['IBOX_PROPS'] = (dict, False)
+                    output.props["IBOX_PROPS"] = (dict, False)
                     # starting from troposphere 3.0 propnames is a set
-                    output.propnames = list(output.propnames) + ['IBOX_PROPS']
+                    output.propnames = list(output.propnames) + ["IBOX_PROPS"]
                     # assign auto_get_props populated obj to IBOX property
                     output.IBOX_PROPS = IBOX_PROPS
-                    output.IBOX_PROPS['MAP'][n] = mapname
+                    output.IBOX_PROPS["MAP"][n] = mapname
                     add_obj(output)
 
             func_map = {
-                'IBOX_PARAMETER': _parameter,
-                'IBOX_CONDITION': _condition,
-                'IBOX_OUTPUT': _output,
+                "IBOX_PARAMETER": _parameter,
+                "IBOX_CONDITION": _condition,
+                "IBOX_OUTPUT": _output,
             }
             for pco in list(func_map.keys()):
                 try:
@@ -568,97 +609,102 @@ def auto_get_props(obj, mapname=None, key=None, rootdict=None):
                 else:
                     func_map[pco](k)
 
-        def _auto_PO(name, conf, mode, value=''):
-            if mode == 'p':
-                parameter_base = {'Description': 'Empty for mapped value'}
+        def _auto_PO(name, conf, mode, value=""):
+            if mode == "p":
+                parameter_base = {"Description": "Empty for mapped value"}
                 parameter_base.update(conf)
-                parameter = {'IBOX_PARAMETER': {
-                    f'{mapname}{name}': parameter_base}}
+                parameter = {"IBOX_PARAMETER": {f"{mapname}{name}": parameter_base}}
                 _try_PCO_in_obj(parameter)
-            if mode == 'o':
-                output_base = {'Value': value}
+            if mode == "o":
+                if isinstance(value, int):
+                    # Output value must be a string
+                    value = str(value)
+                output_base = {"Value": value}
                 output_base.update(conf)
-                output = {'IBOX_OUTPUT': {
-                    f'{mapname}{name}': output_base}}
+                output = {"IBOX_OUTPUT": {f"{mapname}{name}": output_base}}
                 _try_PCO_in_obj(output)
 
         # Allowed object properties
-        props = list(vars(obj)['propnames'])
-        props.extend(vars(obj)['attributes'])
+        props = list(vars(obj)["propnames"])
+        props.extend(vars(obj)["attributes"])
 
         # Parameters, Conditions, Outpus in yaml cfg
         _try_PCO_in_obj(key)
 
-        for propname in key:
-            key_value = key[propname]
+        for propname in dict(
+            list(key.items()) + list(dict.fromkeys(props, None).items())
+        ).keys():
+            # iterate over both obj props and key - using key order
 
-            if propname.endswith(
-                    ('.IBOX_PCO', '.IBOX_AUTO_PO', '.IBOX_AUTO_P')):
-                if propname.endswith('.IBOX_PCO'):
-                    # If there is a key ending with {prop}.IBOX_PCO process it
-                    _try_PCO_in_obj(key_value)
-                if propname.endswith(('.IBOX_AUTO_PO', '.IBOX_AUTO_P')):
+            # IBOX_AUTO_PO
+            for n in [f"{propname}.IBOX_AUTO_PO", f"{propname}.IBOX_AUTO_P"]:
+                if n in key:
                     # Automatically create parameter
-                    _auto_PO(propname.split('.')[0], key_value, 'p')
-                continue
+                    _auto_PO(propname, key[n], "p")
 
-            if propname in props and f'{propname}.IBOX_CODE' not in key:
-                # propname is an obj props and there is no propname key
-                # defining code (look down for processing IBOX_CODE propname).
+            # IBOX_PCO
+            ibox_pco = f"{propname}.IBOX_PCO"
+            if ibox_pco in key:
+                # If there is a key ending with {prop}.IBOX_PCO process it
+                _try_PCO_in_obj(key[ibox_pco])
+
+            # IBOX_CODE
+            ibox_code = f"{propname}.IBOX_CODE"
+            if ibox_code in key:
+                # If there is a key ending with {prop}.IBOX_CODE
+                # eval it and use it as prop value.
+                # Usefull if a str value need to be processed by a code.
+                # (like in autoscaling-scheduledactions.yml)
+                value = eval(key[ibox_code])
+            elif propname in key and propname in props:
+                # there is match between obj prop and a dict key
+                key_value = key[propname]
 
                 # set value
                 if isinstance(key_value, dict):
                     # key value is a dict, get populated object
                     value = _get_obj(obj, key, propname, mapname)
-                elif (isinstance(key_value, str)
-                        and key_value.startswith('IBOX_RESNAME')):
+                elif isinstance(key_value, str) and key_value.startswith(
+                    "IBOX_RESNAME"
+                ):
                     # replace IBOX_RESNAME string with IBOX_RESNAME value
-                    value = key_value.replace('IBOX_RESNAME', IBOX_RESNAME)
+                    value = key_value.replace("IBOX_RESNAME", IBOX_RESNAME)
                 elif rootdict:
                     # rootdict is needed by lib/efs.py EFS_FileStorage SGIExtra
                     # where is passed as a dictionary to parse for parameters
-                    value = get_endvalue(
-                        f'{mapname}{propname}', fixedvalues=rootdict,
-                        resname=IBOX_RESNAME)
-                elif (isinstance(key_value, str)
-                        and key_value.startswith('get_endvalue(')):
+                    value = get_endvalue(f"{mapname}{propname}", fixedvalues=rootdict)
+                elif isinstance(key_value, str) and key_value.startswith(
+                    "get_endvalue("
+                ):
                     # Usefull to migrate code in yaml using auto_get_props
                     # get_endvalue is used only when migrating code
                     value = eval(key_value)
                 else:
-                    value = get_endvalue(
-                        f'{mapname}{propname}', resname=IBOX_RESNAME)
+                    value = get_endvalue(f"{mapname}{propname}")
 
-                if key_value == 'IBOX_IFCONDVALUE':
+                if key_value == "IBOX_IFCONDVALUE":
                     # auto add condition and wrap value in AWS If Condition
-                    add_obj(get_condition(f'{mapname}{propname}',
-                                          'not_equals', 'IBOX_IFCONDVALUE'))
-                    value = If(
-                        f'{mapname}{propname}',
-                        value,
-                        Ref('AWS::NoValue'))
+                    add_obj(
+                        get_condition(
+                            f"{mapname}{propname}", "not_equals", "IBOX_IFCONDVALUE"
+                        )
+                    )
+                    value = If(f"{mapname}{propname}", value, Ref("AWS::NoValue"))
 
                 # trick to wrapper recursed returned value in If Condition
                 try:
-                    if_wrapper = key_value['IBOX_IF']
+                    if_wrapper = key_value["IBOX_IF"]
                 except Exception:
                     pass
                 else:
                     value = _iboxif(if_wrapper, mapname, value)
 
-                if propname == 'Condition' and not isinstance(value, str):
+                if propname == "Condition" and not isinstance(value, str):
                     # Avoid intercepting a Template Condition
                     # as a Resource Condition
                     continue
-            elif propname.endswith('.IBOX_CODE'):
-                # If there is a key ending with {prop}.IBOX_CODE
-                # eval it and use it as prop value.
-                # Usefull if a str value need to be processed by a code.
-                # (like in autoscaling-scheduledactions.yml)
-                value = eval(key_value)
-                propname = propname.replace('.IBOX_CODE', '')
             else:
-                # NO match between propname and one of obj props
+                # NO match between propname and cfg keys
                 continue
 
             # Finally set obj property
@@ -668,17 +714,16 @@ def auto_get_props(obj, mapname=None, key=None, rootdict=None):
                 pass
             else:
                 # populate IBOX_PROPS dict
-                IBOX_PROPS[f'{mapname}{propname}'] = (obj, propname)
+                IBOX_PROPS[f"{mapname}{propname}"] = (obj, propname)
 
                 # Automatically create output
-                for n in [f'{propname}.IBOX_AUTO_PO',
-                          f'{propname}.IBOX_AUTO_O']:
+                for n in [f"{propname}.IBOX_AUTO_PO", f"{propname}.IBOX_AUTO_O"]:
                     if n in key:
-                        _auto_PO(propname, key[n], 'o', value)
+                        _auto_PO(propname, key[n], "o", value)
 
         # title override
         try:
-            obj.title = key['IBOX_TITLE']
+            obj.title = key["IBOX_TITLE"]
         except Exception:
             pass
 
@@ -687,13 +732,13 @@ def auto_get_props(obj, mapname=None, key=None, rootdict=None):
 
 
 def auto_build_obj(obj, key, name=None):
-    props = vars(obj)['propnames']
+    props = vars(obj)["propnames"]
     classname = obj.__class__
     for resname, resvalue in key.items():
         final_obj = classname(resname)
         if not name:
             name = final_obj.__class__.__name__
-        auto_get_props(final_obj, f'{name}{resname}')
+        auto_get_props(final_obj, f"{name}{resname}")
 
         add_obj(final_obj)
 
@@ -703,20 +748,21 @@ def change_obj_data(obj, find, value):
         for n, v in enumerate(obj):
             change_obj_data(obj[n], find, value)
     elif isinstance(obj, If):
-        obj_if = obj.data['Fn::If']
+        obj_if = obj.data["Fn::If"]
         for n, v in enumerate(obj_if):
-            if isinstance(v, Ref) and v.data['Ref'] == find:
+            if isinstance(v, Ref) and v.data["Ref"] == find:
                 obj_if[n] = value
 
 
 def gen_random_string():
     length = 16
-    char_set = f'{string.ascii_letters}{string.digits}'
+    char_set = f"{string.ascii_letters}{string.digits}"
     if not hasattr(gen_random_string, "rng"):
         # Create a static variable
         gen_random_string.rng = random.SystemRandom()
-    secret_string = ''.join(
-        [gen_random_string.rng.choice(char_set) for _ in range(length)])
+    secret_string = "".join(
+        [gen_random_string.rng.choice(char_set) for _ in range(length)]
+    )
 
     return secret_string
 
@@ -729,16 +775,16 @@ def clf_compute_order(pattern):
 
     n_star = 0
     for n, v in enumerate(pattern):
-        if v == '?':
+        if v == "?":
             base_ord += 0.0001
-        elif v == '*':
+        elif v == "*":
             if len(pattern) == n + 1 and n_star != 0:
                 # v is last char but not the only star char
                 base_ord += 0.0005
             elif n_star != 0:
-                base_ord -= 0.000001*n
+                base_ord -= 0.000001 * n
             else:
-                base_ord += 1000/n
+                base_ord += 1000 / n
                 n_star += 1
         elif n_star != 0:
             base_ord -= 0.001
@@ -746,3 +792,21 @@ def clf_compute_order(pattern):
     cfg.dbg_clf_compute_order[pattern] = base_ord
 
     return base_ord
+
+
+def camel_to_snake(data):
+    out = ""
+    skip_next = False
+    for n, v in enumerate(data):
+        if skip_next:
+            skip_next = False
+            continue
+        if v.isupper() and data[n + 1].isupper():
+            out += f"_{v}{data[n+1]}"
+            skip_next = True
+        elif v.isupper():
+            out += f"_{v}" if out else v
+        else:
+            out += v
+
+    return out.upper()
